@@ -1,4 +1,5 @@
 from ...__private.utilities import *
+from ... import Event
 
 
 class DataStore(ABC):
@@ -7,7 +8,7 @@ class DataStore(ABC):
         raise NotImplementedError()
 
     @abstractmethod
-    def set_value(self, key: str, value: Any) -> None:
+    def set_value(self, key: str, value: Any, cause: Event = None) -> None:
         raise NotImplementedError()
 
     @abstractmethod
@@ -25,7 +26,7 @@ class DataStore(ABC):
 
     def get_branch(self, prefix: str,
                    value_generator: Callable[[str], Any] = None,
-                   change_event_handlers: List[Callable[[str, Optional[Any], Any], None]] = None,
+                   change_event_handlers: List[Callable[[str, Optional[Any], Any, Optional[Event]], None]] = None,
                    change_indicator: Callable[[Any, Any], bool] = lambda x, y: x != y
                    ):
         return DataStoreBranch(self, prefix, value_generator, change_event_handlers, change_indicator)
@@ -36,7 +37,7 @@ class DataStoreBranch(DataStore):
                  parent_data_store: DataStore,
                  prefix: str,
                  value_generator: Callable[[str], Any],
-                 change_event_handlers: List[Callable[[str, Optional[Any], Any], None]],
+                 change_event_handlers: List[Callable[[str, Optional[Any], Any, Optional[Event]], None]],
                  change_indicator: Callable[[Any, Any], bool]
                  ):
         super().__init__()
@@ -56,17 +57,17 @@ class DataStoreBranch(DataStore):
                     handler(key, None, result)
         return result
 
-    def set_value(self, key: str, value: Any) -> None:
+    def set_value(self, key: str, value: Any, cause: Event = None) -> None:
         should_call_handlers = False
         if self._change_event_handlers is not None and len(self._change_event_handlers) > 0:
-            should_call_handlers = True
             previous_value = self.get_value(key)
+            should_call_handlers = True
             if not self._change_indicator(previous_value, value):
                 return
-        self._parent_data_store.set_value(self._prefix + key, value)
+        self._parent_data_store.set_value(self._prefix + key, value, cause)
         if should_call_handlers:
             for handler in self._change_event_handlers:
-                handler(key, previous_value, value)
+                handler(key, previous_value, value, cause)
 
     def contains(self, key: str) -> bool:
         return self._parent_data_store.contains(self._prefix + key)
@@ -80,7 +81,7 @@ class SimpleRAMDataStore(DataStore):
     def get_value(self, key: str) -> Optional[Any]:
         return self._storage.get(key, None)
 
-    def set_value(self, key: str, value: Any) -> None:
+    def set_value(self, key: str, value: Any, cause: Event = None) -> None:
         self._storage[key] = value
 
     def contains(self, key: str) -> bool:
